const TestLogic = require('./test.logic');
const ReportServerLogic = require('../report-server/report-server.logic');
const chalk = require('chalk');
const SdkHolder = require('../../../libs/sdk/sdk.holder');
const _ = require('lodash');
const { Config: sdkConfig } = require('codefresh-sdk');
const fileManager = require('../../../libs/file-manager');
const config = require('../../../config');
const path = require('path');

class TestController {

    static async _resolveAccessParams({ context, userToken, adminToken, url }) {

        if(context) {
            const config = await sdkConfig.fromCodefreshConfig();
            return { url: config.context.url, token: config.context.token };
        }

        if(userToken) {
            return { url, token: userToken };
        }

        if(adminToken) {
            return { url, token: adminToken };
        }

        throw Error('Provide user token');
    }

    /**
     * Create in this project dir containing steps definitions before they will be run
     * Before creating need to clear old directory to avoid some errors
     * @returns {Promise<void>}
     */
    static #resolveSteps = async () => {
        try {
            await fileManager.removeDir({path: path.resolve(process.cwd(), config.getStepsPath())});
            await fileManager.copyDir({from: config.getSourceStepsPath(), to: path.resolve(process.cwd(), config.getStepsPath())})
        } catch (e) {
            throw new Error(`Cant resolve steps definitions, cased by: ${e.message}`);
        }
    };

    static async runTests(argv) {
        config.init({ argv });
        await this.#resolveSteps();

        const { adminToken, userToken, apiUrl, reportServer, port, tag: tags, coverage, reportPath } = argv;
        let report = argv.report;

        if (argv.report === '') {
            report = 'html';
        }

        if(reportPath) {
            config.setReportPath(reportPath);
        }

        const context = argv.context === true;

        let createdUserInfo = {};

        if (!adminToken && !userToken && !context) {
            throw new Error('You must provide type of auth, ex: --admin-token or --user-token or --context');
        }

        try {
            console.log(chalk.green('Start cucumber tests'));

            const { url, token } = await this._resolveAccessParams({ context, userToken, adminToken, url: apiUrl });

            let newUser = false;

            const allowRunTest = await TestLogic.allowRunTests({ apiUrl: url, token });
            if(!allowRunTest) {
                throw Error('Can`t run tests on codefresh account');
            }

            if (context) {
                await SdkHolder.initSdk({ context });
            }

            if (userToken) {
                await SdkHolder.initSdk({ userToken, baseEndpoint: url });
            }

            if (adminToken) {
                const { token, userName, accountId } = await TestLogic.getNewUser({
                    apiUrl,
                    adminToken,
                });
                createdUserInfo = {
                    userName,
                    accountId,
                };
                await SdkHolder.initSdk({ userToken: token, baseEndpoint: url });
                newUser = true;
            }

            const result = await TestLogic.runCucumber({ tags, newUser });

            if (adminToken) {
                await TestLogic.clearCreatedAccountAndUser({
                    ...createdUserInfo,
                    apiUrl,
                    adminToken,
                });
            }

            if (coverage) {
                this.#showCoverage();
            }

            if (report) {
                await this.#generateReport({ reportServer, port, report });
            }

            if(!_.get(result, 'success')) {
                process.exit(1);
            }
        } catch (e) {
            throw e;
        }
    }

    static #showCoverage = () => {
        const coverage = Math.round((SdkHolder.usedMethods.length / SdkHolder.allMethods.length) * 100);

        _.forEach(SdkHolder.usedMethods, item => {
            console.log(chalk.green('+ ' + item));
        });

        _.forEach(_.difference(SdkHolder.allMethods, SdkHolder.usedMethods), item => {
            console.log(chalk.red('- ' + item));
        });

        console.log(chalk.blue(`Coverage: ${coverage}%`));
    };

    static #generateReport = async ({ reportServer, port, report }) => {
        console.log(chalk.green('Start allure report generate\n'));
        if (report === 'allure') {
            await TestLogic.generateAllureReport();
        }

        if (report === 'html') {
            await TestLogic.generateHtmlReport();
        }

        if (reportServer) {
            ReportServerLogic.startServer(port, report + '-report');
            console.log(chalk.green(`Report server started at :${port}\n`));
        }
    };
}

module.exports = TestController;
