const { Cli: CocumberCli } = require('cucumber');
const allure = require('allure-commandline');
const Api = require('../../../libs/api');
const randomstring = require('randomstring');
const htmlReporter = require('cucumber-html-reporter');
const config = require('../../../config');

const CODEFRESH_INC_ACCOUNT = 'codefresh-inc';

class TestLogic {
    static generateAllureReport() {
        return new Promise((resolve) => {
            const child = allure([
                'generate',
                `${config.getReportPath()}/allure-results/`,
                '--clean',
                '-o',
                `${config.getReportPath()}/allure-report`
            ]);

            child.on('exit', () => {
                return resolve();
            });
        });
    };

    static generateHtmlReport() {
        const options = {
            theme: 'bootstrap',
            jsonFile: './report.json',
            output: './html-report/index.html',
            reportSuiteAsScenarios: true,
            launchReport: false,
            metadata: {
                'App': 'Codefresh',
            },
        };

        htmlReporter.generate(options);

    }

    static #featuresFilePath = (newUser) => {
        if(newUser) {
            return config.getSourceFeaturesNewUserPath();
        }
        return config.getSourceFeaturesExistsUserPath();
    };

    static runCucumber({ tags, newUser }) {


        const cli = new CocumberCli({
            argv: [
                process.argv[0],
                process.argv[1],
                this.#featuresFilePath(newUser),
                 ...this.#prepareTags(tags),
                '--require',
                './steps/**/*.js',
                '-f',
                './allure.reporter.js:/dev/null',
                '-f',
                'progress',
                '-f',
                'json:./report.json',

            ],
            cwd: process.cwd(),
            stdout: process.stdout
        });

        return cli.run();
    }

    static #prepareTags = (tags) => {
        if (!tags.length) {
            return [];
        }

        const tagExpression = tags.map(item => '@' + item).join(' or ');

        return [
            '-t',
            `${tagExpression}`,
        ];
    };

    static async getNewUser({ apiUrl, adminToken }) {

        const api = new Api({
            endpoint: apiUrl,
            token: adminToken,
        });

        const accountName = randomstring.generate();
        const userMail = `${accountName}@gmail.com`;

        const account = await api.createAccount(accountName);
        await api.addUserToAccount(account._id, userMail);
        const user = await api.getUserByUserMail(userMail);
        await api.activeUser(user._id);
        await api.markAsAdmin(account._id.toString(), user._id.toString());

        return {
            token: await api.getAccessTokenByUserId(user._id),
            userName: user.userName,
            accountId: account._id,
        };
    }

    static async allowRunTests({ apiUrl, token }) {
        const api = new Api({
            endpoint: apiUrl,
            token,
        });
        const user = await api.getUser();
        return user.activeAccountName !== CODEFRESH_INC_ACCOUNT;
    }

    static clearCreatedAccountAndUser({ apiUrl, adminToken, accountId, userName }) {
        const api = new Api({
            endpoint: apiUrl,
            token: adminToken,
        });


        return Promise.all([api.deleteAccount(accountId), api.deleteUser(userName)]);
    }
}

module.exports = TestLogic;
